<?php

namespace App\Http\Controllers;

use Validator;
use App\User;
use Firebase\JWT\JWT;
use Illuminate\Http\Request;
use Firebase\JWT\ExpiredException;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Str;
use Laravel\Lumen\Routing\Controller as BaseController;
use Carbon\Carbon;
use DB;
use Illuminate\Support\Facades\Storage;
// use Illuminate\Http\Request;
use App\Page;
use App\Product;
use App\BookingTrip;
use App\PageGroup;
use App\UserGroup;
use App\UserGroupPermission;
use App\SystemUser;
use App\Merchant;
use App\Customer;
use App\Point;
use App\PointTransaction;
use App\MerchantService;
use App\ServiceImage;
use App\SalesTransaction;
use App\BusinessType;
use App\Advert;

class MerchantController extends BaseController 
{
    /**
     * The request instance.
     *
     * @var \Illuminate\Http\Request
     */
    private $request;

    /**
     * Create a new controller instance.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return void
     */
    public function __construct(Request $request) {
        $this->request = $request;
    }

  

 /// merchant_id function start
 public function getAllMerchantServices(){
        $dataList = MerchantService::All();
        if($dataList){
            $dataListRecord=array();
            $images = ServiceImage::All();
            foreach ($dataList as $row) {
                $imagesArr=array();
                $rowArr=array();
                foreach ($images as $image_record) {
                  if($row->service_id == $image_record->service_id){
                    array_push($imagesArr, $image_record->image_link);
                  }
                }
                # code...
                array_push($rowArr,$row);
                $imageRec = array('images'=>$imagesArr);
                $merged_result=array_merge($rowArr, $imageRec);
                array_push($dataListRecord, $merged_result);
            }
            return response()->json([
                'status' => 'success', 'result' => $dataListRecord], 200);
        }else{
             return response()->json([
                'status' => 'fail', 'result' => ""], 200);
        }
    }

public function getAllActiveMerchants(){
        $dataList = Merchant::where('status','active')->get();
        if($dataList){
            return response()->json([
                'status' => 'success', 'result' => $dataList], 200);
        }else{
             return response()->json([
                'status' => 'fail', 'result' => ""], 200);
        }
    }
  

    
    public function getMerchantApprovedSales(Request $request){
        $merchant_id = $request->input('merchant_id');

        $dataList = SalesTransaction::join('merchants', 'merchants.merchant_id', '=','sales_transactions.merchant_id')
        ->join('customers', 'customers.customer_id', '=','sales_transactions.customer_id')
        ->where('sales_transactions.merchant_id',$merchant_id)
        ->where('sales_transactions.transaction_approved','yes')
        ->select("sales_transactions.*","merchants.*","customers.*")->orderBy('sales_transactions.date_initiated', 'asc')->get();
        if($dataList){
            return response()->json([
                'status' => 'success', 'result' => $dataList], 200);
        }else{
           return response()->json([
            'status' => 'fail', 'result' => ""], 200);
       }
   }

   public function getMerchantSalesPending(Request $request){
    $merchant_id = $request->input('merchant_id');
    $dataList = SalesTransaction::join('merchants', 'merchants.merchant_id', '=','sales_transactions.merchant_id')
    ->join('customers', 'customers.customer_id', '=','sales_transactions.customer_id')
    ->where('sales_transactions.merchant_id',$merchant_id)
    ->where('sales_transactions.transaction_approved','no')
    ->select("sales_transactions.*","merchants.*","customers.*")->orderBy('sales_transactions.date_initiated', 'asc')->get();
    if($dataList){
        return response()->json([
            'status' => 'success', 'result' => $dataList], 200);
    }else{
      return response()->json([
        'status' => 'fail', 'result' => ""], 200);
  }
}

    public function approveSale(Request $request)
    { 
      
        $sale_id = $request->input('sale_transaction_id');
        
       $dataModel = DB::table('sales_transactions')->where('sale_transaction_id',$sale_id)->update(
            [
             
             'transaction_approved' => 'yes',
             'date_approved' => Carbon::now(),
         ]
        );
        
         if($dataModel){

            // $new_record = Product::where('merchant_id_function_id',$record_id)->first();
              $sData = SalesTransaction::join('merchants', 'merchants.merchant_id', '=','sales_transactions.merchant_id')
  ->join('customers', 'customers.customer_id', '=','sales_transactions.customer_id')
  ->where('sale_transaction_id',$sale_id)->first();
  $this->updateCustomerPoint($sData->customer_id,doubleval(($sData->commission_to_customer/100)));
            return response()->json([
                'status' => 'success', 'result' => $sData,'message' => "Sale Approved Successfully"], 200);
        }else{
             return response()->json([
                'status' => 'fail', 'result' => "",'message' => "Sale Not Approved"], 200);
        }
        // } 
        
    }

    public function updateCustomerPoint($customer_id,$point){
     $pointData = Point::where('user_type_id',$customer_id)->where('user_type','customer')->first();
     if($pointData){
        $dataModel = DB::table('points')->where('user_type_id',$customer_id)->where('user_type','customer')->update(
            [
             'point' => ($point + $pointData->point),
             'updated_at' => Carbon::now(),
         ]
        );
        $this->addPointGainTransaction($customer_id,$point,$pointData->point,($point + $pointData->point),'Points gain from purchase','point_gain');
     }
     
    }

    public function addPointGainTransaction($customer_id,$point,$previous_point,$current_point,$reference,$transaction_type){
       
       $pointData = new PointTransaction();
       $pointData->user_type = 'customer';
       $pointData->user_type_id = $customer_id;
       $pointData->point = $point;
       $pointData->previous_point = $previous_point;
       $pointData->current_point = $current_point;
       $pointData->reference = $reference;
       $pointData->transaction_type = $transaction_type;
       $pointData->save();
    }

 // addition of products by merchants

  public function getAllProducts(){
        $dataList = MerchantService::All();
        if($dataList){
            return response()->json([
                'status' => 'success', 'result' => $dataList], 200);
        }else{
             return response()->json([
                'status' => 'fail', 'result' => ""], 200);
        }
    }
 public function getAllAdvertsByMerchant(Request $request){
        $merchant_id = $request->input('merchant_id');
        $dataList = Advert::where('merchant_id',$merchant_id)->get();
        if($dataList){
            return response()->json([
                'status' => 'success', 'result' => $dataList], 200);
        }else{
             return response()->json([
                'status' => 'fail', 'result' => ""], 200);
        }
    }

    public function getAllProductsByMerchant(Request $request){
        $merchant_id = $request->input('merchant_id');
        $dataList = MerchantService::where('merchant_id',$merchant_id)->get();
        if($dataList){
            return response()->json([
                'status' => 'success', 'result' => $dataList], 200);
        }else{
             return response()->json([
                'status' => 'fail', 'result' => ""], 200);
        }
    }

    public function storeImage64($image_64){
      // $image_64 = $data['photo']; //your base64 encoded data
  $extension = explode('/', explode(':', substr($image_64, 0, strpos($image_64, ';')))[1])[1];   // .jpg .png .pdf
  $replace = substr($image_64, 0, strpos($image_64, ',')+1); 
// find substring fro replace here eg: data:image/png;base64,
 $image = str_replace($replace, '', $image_64); 
 $image = str_replace(' ', '+', $image); 
 $imageName = Str::random(10).'.'.$extension;
 Storage::disk('public')->put($imageName, base64_decode($image));
  return $imageName;
    }
    
    public function saveProduct(Request $request)
    { 
        $merchant_id = $request->input('merchant_id');
        $name = $request->input('name');
        $cost = $request->input('cost');
        $description = $request->input('description');
        $business_type_id = $request->input('business_type_id');
  $image_64 = $request->input('file'); //your base64 encoded data
  $image_base = $request->input('image_base'); 
        
        $check_existence = MerchantService::where('merchant_id',$merchant_id)->where('name',$name)->where('cost',$cost)->where('description',$description)->first();
        if($check_existence){// check for existence before saveing
            return response()->json([
                'status' => 'fail', 'result' => "",'message' => "Record Already Exist"], 200);
        }else{// save and return data

        if($image_64 == ''){
            $imageName = "";
        }else{
            $imageName = $this->storeImage64($image_64);
        }
        

        $dataBusinessType = BusinessType::where('business_type_id',intval($business_type_id))->first();
        $dataModel = new MerchantService();
        $dataModel->merchant_id = $merchant_id;
        $dataModel->name = $name;
        $dataModel->cost = $cost;
        $dataModel->description = $description;
         if($image_64 != ''){
        $dataModel->image_link = $image_base."app/public/".$imageName;
    }
        $dataModel->business_type_id = $business_type_id;
        $dataModel->business_type_name = $dataBusinessType->category;
        $dataModel->category = $dataBusinessType->category;
        $dataModel->updated_at = Carbon::now();
        $dataModel->created_at = Carbon::now();
        $dataModel->save();
         if($dataModel){
            $new_record = MerchantService::where('merchant_id',$merchant_id)->where('name',$name)->where('cost',$cost)->where('description',$description)->first();
            return response()->json([
                'status' => 'success', 'result' => $new_record,'message' => "Product Added Successfully"], 200);
        }else{
             return response()->json([
                'status' => 'fail', 'result' => "",'message' => "Product Not Added"], 200);
        }
        }
        
    }

    public function updateProduct(Request $request)
    { 
      $service_id = $request->input('service_id');
      $name = $request->input('name');
      $merchant_id = $request->input('merchant_id');
        $cost = $request->input('cost');
        $description = $request->input('description');
        $business_type_id = $request->input('business_type_id');
        $dataBusinessType = BusinessType::where('business_type_id',$business_type_id)->first();
        $image_64 = $request->input('file'); //your base64 encoded data
  $image_base = $request->input('image_base'); 
  if($image_64 == ''){
            $imageName = "";
        }else{
            $imageName = $this->storeImage64($image_64);
        }
        if($image_64 == ''){
 $dataModel = DB::table('merchant_services')->where('service_id',$service_id)->update(
            [
             'name' => $name,
             'cost' => $cost,
             'description' => $description,
             'business_type_id' => $business_type_id,
             'business_type_name' => $dataBusinessType->category,
             'category' => $dataBusinessType->category,
             'updated_at' => Carbon::now(),
         ]
        );
             }else{
 $dataModel = DB::table('merchant_services')->where('service_id',$service_id)->update(
            [
             'name' => $name,
             'cost' => $cost,
             'description' => $description,
             'business_type_id' => $business_type_id,
             'image_link' => $image_base."app/public/".$imageName,
             'business_type_name' => $dataBusinessType->category,
             'category' => $dataBusinessType->category,
             'updated_at' => Carbon::now(),
         ]
        );
             }
      
        
         if($dataModel){
            // $new_record = Product::where('merchant_id_function_id',$record_id)->first();
              $new_record= MerchantService::where([
                ['service_id',$service_id],
            ])
            ->select("*")->first();
            return response()->json([
                'status' => 'success', 'result' => $new_record,'message' => "Product / Service Updated Successfully"], 200);
        }else{
             return response()->json([
                'status' => 'fail', 'result' => "",'message' => "Product / Service Not Updated"], 200);
        }
        // } 
        
    }
  

    public function deleteProduct(Request $request)
    { 
        $record_id = $request->input('service_id');
        $dataModel = MerchantService::where('service_id',$record_id)->delete();
        if($dataModel){
            return response()->json([
                'status' => 'success', 'result' => "",'message' => "Product Deleted"], 200);
        }else{
         return response()->json([
            'status' => 'fail', 'result' => "",'message' => "Product Not Deleted"], 200);
     }
 }

}

