<?php

namespace App\Http\Controllers;

use Validator;
use App\User;
use Firebase\JWT\JWT;
use Illuminate\Http\Request;
use Firebase\JWT\ExpiredException;
use Illuminate\Support\Facades\Hash;
use Laravel\Lumen\Routing\Controller as BaseController;
use Carbon\Carbon;
use DB;
// use Illuminate\Http\Request;
use App\Page;
use App\PageFunction;
use App\PageGroup;
use App\UserGroup;
use App\UserGroupPermission;


class PermissionController extends BaseController 
{
    /**
     * The request instance.
     *
     * @var \Illuminate\Http\Request
     */
    private $request;

    /**
     * Create a new controller instance.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return void
     */
    public function __construct(Request $request) {
        $this->request = $request;
    }

    public function getAllPageGroups(){
        $pageGroups = PageGroup::all();
        if($pageGroups){
            return response()->json([
                'status' => 'success', 'result' => $pageGroups], 200);
        }else{
             return response()->json([
                'status' => 'fail', 'result' => ""], 200);
        }
    }

    public function savePageGroup(Request $request)
    { 
        $page_group_name = $request->input('page_group_name');
        $group_icon = $request->input('group_icon');
        $check_existence = PageGroup::where('page_group_name',$page_group_name)->where('group_icon',$group_icon)->first();
        if($check_existence){// check for existence before saveing
            return response()->json([
                'status' => 'fail', 'result' => "",'message' => "Record Already Exist"], 200);
        }else{// save and return data
        $dataModel = new pageGroup();
        $dataModel->page_group_name = $page_group_name;
        $dataModel->group_icon = $group_icon;
        $dataModel->updated_at = Carbon::now();
        $dataModel->created_at = Carbon::now();
        $dataModel->save();
         if($dataModel){
            $new_record = PageGroup::where('page_group_name',$page_group_name)->where('group_icon',$group_icon)->first();
            return response()->json([
                'status' => 'success', 'result' => $new_record,'message' => "Page Group Saved Successfully"], 200);
        }else{
             return response()->json([
                'status' => 'fail', 'result' => "",'message' => "Record Not Saved"], 200);
        }
        }
        
    }

    public function updatePageGroup(Request $request)
    { 
        $page_group_name = $request->input('page_group_name');
        $group_icon = $request->input('group_icon');
        $record_id = $request->input('page_group_id');
        $check_existence = PageGroup::where('page_group_name',$page_group_name)->where('group_icon',$group_icon)->first();
        if($check_existence){// check for existence before saveing
            return response()->json([
                'status' => 'fail', 'result' => "",'message' => "Record Already Exist With Details Provided"], 200);
        }else{// save and return data
        // $dataModel =  PageGroup::where('page_group_id',$record_id)->first();
       $dataModel = DB::table('page_groups')->where('page_group_id',$record_id)->update(
            [
                'page_group_name' => $page_group_name,
             'group_icon' => $group_icon,
             'updated_at' => Carbon::now(),
         ]
        );
        
         if($dataModel){
            $new_record = PageGroup::where('page_group_id',$record_id)->first();
            return response()->json([
                'status' => 'success', 'result' => $new_record,'message' => "Page Group Updated Successfully"], 200);
        }else{
             return response()->json([
                'status' => 'fail', 'result' => "",'message' => "Record Not Saved"], 200);
        }
        } 
        
    }

    public function deletePageGroup(Request $request)
    { 
        $record_id = $request->input('page_group_id');
        $dataModel = PageGroup::where('page_group_id',$record_id)->delete();
        if($dataModel){
            return response()->json([
                'status' => 'success', 'result' => "",'message' => "Page Group Deleted"], 200);
        }else{
         return response()->json([
            'status' => 'fail', 'result' => "",'message' => "Page Group Not Deleted"], 200);
     }
 }
    // page group ends

    /// pages start
 public function getAllPages(){
        $pages = Page::join('page_groups', 'page_groups.page_group_id', '=','pages.page_group' )
            ->select("*")->orderBy('pages.page_id', 'asc')->get();
        if($pages){
            return response()->json([
                'status' => 'success', 'result' => $pages], 200);
        }else{
             return response()->json([
                'status' => 'fail', 'result' => ""], 200);
        }
    }
    public function savePage(Request $request)
    { 
        $page_name = $request->input('page_name');
        $page_group = $request->input('page_group');
        $icon = $request->input('icon');
        $read_link = $request->input('read_link');
        $check_existence = Page::where('page_name',$page_name)->where('page_group',$page_group)->where('read_link',$read_link)->where('icon',$icon)->first();
        if($check_existence){// check for existence before saveing
            return response()->json([
                'status' => 'fail', 'result' => "",'message' => "Record Already Exist"], 200);
        }else{// save and return data
        $dataModel = new page();
        $dataModel->page_name = $page_name;
        $dataModel->icon = $icon;
        $dataModel->page_group = $page_group;
        $dataModel->read_link = $read_link;
        $dataModel->updated_at = Carbon::now();
        $dataModel->created_at = Carbon::now();
        $dataModel->save();
         if($dataModel){
            $new_record = Page::where('page_name',$page_name)->where('page_group',$page_group)->where('read_link',$read_link)->where('icon',$icon)->first();
            return response()->json([
                'status' => 'success', 'result' => $new_record,'message' => "Page Saved Successfully"], 200);
        }else{
             return response()->json([
                'status' => 'fail', 'result' => "",'message' => "Page Not Saved"], 200);
        }
        }
        
    }

    public function updatePage(Request $request)
    { 
          $page_name = $request->input('page_name');
        $page_group = $request->input('page_group');
        $icon = $request->input('icon');
        $read_link = $request->input('read_link');
        $record_id = $request->input('page_id');
        $check_existence = Page::where('page_name',$page_name)->where('page_group',$page_group)->where('read_link',$read_link)->where('icon',$icon)->first();
        if($check_existence){// check for existence before saveing
            return response()->json([
                'status' => 'fail', 'result' => "",'message' => "Record Already Exist With Details Provided"], 200);
        }else{// save and return data
        // $dataModel =  PageGroup::where('page_group_id',$record_id)->first();
       $dataModel = DB::table('pages')->where('page_id',$record_id)->update(
            [
                'page_name' => $page_name,
             'page_group' => $page_group,
             'icon' => $icon,
             'read_link' => $read_link,
             'updated_at' => Carbon::now(),
         ]
        );
        
         if($dataModel){
            // $new_record = Page::where('page_id',$record_id)->first();
            $new_record= Page::join('page_groups', 'page_groups.page_group_id', '=','pages.page_group' )
            ->where([
                ['pages.page_id',$record_id],
            ])
            ->select("*")->first();
            return response()->json([
                'status' => 'success', 'result' => $new_record,'message' => "Page Updated Successfully"], 200);
        }else{
             return response()->json([
                'status' => 'fail', 'result' => "",'message' => "Page Not Updated"], 200);
        }
        } 
        
    }

    public function deletePage(Request $request)
    { 
        $record_id = $request->input('page_id');
        $dataModel = Page::where('page_id',$record_id)->delete();
        if($dataModel){
            return response()->json([
                'status' => 'success', 'result' => "",'message' => "Page Deleted"], 200);
        }else{
         return response()->json([
            'status' => 'fail', 'result' => "",'message' => "Page Not Deleted"], 200);
     }
 }

 // page endes

 /// page function start
 public function getAllPageFunctions(){
        $dataList = PageFunction::join('pages', 'pages.page_id', '=','page_functions.page')
             ->select("*")->orderBy('page_functions.page_function_id', 'asc')->get();
        if($dataList){
            return response()->json([
                'status' => 'success', 'result' => $dataList], 200);
        }else{
             return response()->json([
                'status' => 'fail', 'result' => ""], 200);
        }
    }
    public function getAllPageFunctionsJoinOthers(){
        $dataList = PageFunction::join('pages', 'pages.page_id', '=','page_functions.page')
            ->join('page_groups', 'page_groups.page_group_id', '=','pages.page_group' )
             ->select("*")->orderBy('page_functions.page_function_id', 'asc')->get();
        if($dataList){
            return response()->json([
                'status' => 'success', 'result' => $dataList], 200);
        }else{
             return response()->json([
                'status' => 'fail', 'result' => ""], 200);
        }
    }
    public function savePageFunction(Request $request)
    { 
        $page = $request->input('page');
        $function_name = $request->input('function_name');
        $post_link = $request->input('post_link');
        $check_existence = PageFunction::where('page',$page)->where('function_name',$function_name)->where('post_link',$post_link)->first();
        if($check_existence){// check for existence before saveing
            return response()->json([
                'status' => 'fail', 'result' => "",'message' => "Record Already Exist"], 200);
        }else{// save and return data
        $dataModel = new PageFunction();
        $dataModel->page = $page;
        $dataModel->function_name = $function_name;
        $dataModel->post_link = $post_link;
        $dataModel->updated_at = Carbon::now();
        $dataModel->created_at = Carbon::now();
        $dataModel->save();
         if($dataModel){
            $new_record = PageFunction::where('page',$page)->where('function_name',$function_name)->where('post_link',$post_link)->first();
            return response()->json([
                'status' => 'success', 'result' => $new_record,'message' => "Page function Saved Successfully"], 200);
        }else{
             return response()->json([
                'status' => 'fail', 'result' => "",'message' => "Page Not Saved"], 200);
        }
        }
        
    }

    public function updatePageFunction(Request $request)
    { 
          $page = $request->input('page');
        $function_name = $request->input('function_name');
        $post_link = $request->input('post_link');
        $record_id = $request->input('page_function_id');
        $check_existence = PageFunction::where('page',$page)->where('function_name',$function_name)->where('post_link',$post_link)->first();
        if($check_existence){// check for existence before saveing
            return response()->json([
                'status' => 'fail', 'result' => "",'message' => "Record Already Exist With Details Provided"], 200);
        }else{// save and return data
        // $dataModel =  PageGroup::where('page_group_id',$record_id)->first();
       $dataModel = DB::table('page_functions')->where('page_function_id',$record_id)->update(
            [
             'page' => $page,
             'function_name' => $function_name,
             'post_link' => $post_link,
             'updated_at' => Carbon::now(),
         ]
        );
        
         if($dataModel){
            // $new_record = PageFunction::where('page_function_id',$record_id)->first();
              $new_record= PageFunction::join('pages', 'pages.page_id', '=','page_functions.page')
            ->where([
                ['page_function_id',$record_id],
            ])
            ->select("*")->first();
            return response()->json([
                'status' => 'success', 'result' => $new_record,'message' => "Page Function Updated Successfully"], 200);
        }else{
             return response()->json([
                'status' => 'fail', 'result' => "",'message' => "Page Function Not Updated"], 200);
        }
        } 
        
    }

    public function deletePageFunction(Request $request)
    { 
        $record_id = $request->input('page_function_id');
        $dataModel = PageFunction::where('page_function_id',$record_id)->delete();
        if($dataModel){
            return response()->json([
                'status' => 'success', 'result' => "",'message' => "Page Function Deleted"], 200);
        }else{
         return response()->json([
            'status' => 'fail', 'result' => "",'message' => "Page Function Not Deleted"], 200);
     }
 }

 // page function ends

 /// user start
 public function getAllUsers(){
        // $dataList = User::all();
        $dataList = User::join('user_groups', 'user_groups.user_group_id', '=','users.user_group')
             ->select("*")->orderBy('users.id', 'asc')->get();
        if($dataList){
            return response()->json([
                'status' => 'success', 'result' => $dataList], 200);
        }else{
             return response()->json([
                'status' => 'fail', 'result' => ""], 200);
        }
    }
    public function saveUser(Request $request)
    { 
        $first_name = $request->input('first_name');
        $last_name = $request->input('last_name');
        $email = $request->input('email');
        $password = $password = Hash::make($request->input('password'));
        $user_group = $request->input('user_group');
        $check_existence = User::where('email',$email)->first();
        if($check_existence){// check for existence before saveing
            return response()->json([
                'status' => 'fail', 'result' => "",'message' => "User Email Already Exist"], 200);
        }else{// save and return data
            //process the new number
             $from_id_counts = DB::table('id_counts')->where('name','backend-user')->first();
            $next_number = intval($from_id_counts->value) + 1;
            $zeros = 'busr';
            $length_number = count(str_split(strval($next_number)));
            for ($j = 1; $j <= 6; $j++) {// loop nun char
                if ($length_number == $j) {// if lenth is j
                    for ($i = 0; $i < 6 - $j; $i++) {//while i is less than num char + j // thus create zero strings such that the two length is 10
                        $zeros = $zeros . '0';
                    }
                    $formatted_id = $zeros . $next_number; // attach zero to the number
                    $j = 6; //to end loop
                }
            }

        $dataModel = new User();
        $dataModel->first_name = $first_name;
        $dataModel->last_name = $last_name;
        $dataModel->email = $email;
        $dataModel->formatted_id = $formatted_id;
        $dataModel->user_group = $user_group;
        $dataModel->password = $password;
        $dataModel->updated_at = Carbon::now();
        $dataModel->created_at = Carbon::now();
        $dataModel->save();
         if($dataModel){
            // $new_record = User::where('email',$email)->first();
            $update_id = DB::table('id_counts')->where('name','backend-user')->update(
            [
             'value' => $next_number,
             
         ]
        );
            $new_record = User::join('user_groups', 'user_groups.user_group_id', '=','users.user_group')->where('email',$email)->first();
            return response()->json([
                'status' => 'success', 'result' => $new_record,'message' => "User Saved Successfully"], 200);
        }else{
             return response()->json([
                'status' => 'fail', 'result' => "",'message' => "User Not Saved"], 200);
        }
        }
        
    }

    public function updateUser(Request $request)
    { 
        $first_name = $request->input('first_name');
        $last_name = $request->input('last_name');
        $email = $request->input('email');
        $password = $password = Hash::make($request->input('password'));
        $user_group = $request->input('user_group');
        $record_id = $request->input('id');
        // $dataModel =  PageGroup::where('page_group_id',$record_id)->first();
       $dataModel = DB::table('users')->where('id',$record_id)->update(
            [
             'first_name' => $first_name,
             'last_name' => $last_name,
             'user_group' => $user_group,
             'password' => $password,
             'updated_at' => Carbon::now(),
         ]
        );
        
         if($dataModel){
           $new_record = User::join('user_groups', 'user_groups.user_group_id', '=','users.user_group')->where('id',$record_id)->first();
            // $new_record = User::where('id',$record_id)->first();
            return response()->json([
                'status' => 'success', 'result' => $new_record,'message' => "User Updated Successfully"], 200);
        }else{
             return response()->json([
                'status' => 'fail', 'result' => "",'message' => "User Not Updated"], 200);
        }
         
        
    }

     public function updateUserProfile(Request $request)
    { 
        $first_name = $request->input('first_name');
        $last_name = $request->input('last_name');
        $email = $request->input('email');
        $password = $password = Hash::make($request->input('password'));
        $user_group = $request->input('user_group');
        $record_id = $request->input('id');
        // $dataModel =  PageGroup::where('page_group_id',$record_id)->first();
       $dataModel = DB::table('users')->where('id',$record_id)->update(
            [
             'first_name' => $first_name,
             'last_name' => $last_name,
             'password' => $password,
             'updated_at' => Carbon::now(),
         ]
        );
        
         if($dataModel){
           $new_record = User::join('user_groups', 'user_groups.user_group_id', '=','users.user_group')->where('id',$record_id)->first();
            // $new_record = User::where('id',$record_id)->first();
            return response()->json([
                'status' => 'success', 'result' => $new_record,'message' => "User Updated Successfully"], 200);
        }else{
             return response()->json([
                'status' => 'fail', 'result' => "",'message' => "User Not Updated"], 200);
        }
         
        
    }

     public function saveUserImage(Request $request)
    { 
        $profile_image = $request->input('profile_image');
        $record_id = $request->input('record_id');
        // $dataModel =  PageGroup::where('page_group_id',$record_id)->first();
       $dataModel = DB::table('users')->where('id',$record_id)->update(
            [
             'profile_image' => $profile_image,
             'updated_at' => Carbon::now(),
         ]
        );
        
         if($dataModel){
           // $new_record = User::join('user_groups', 'user_groups.user_group_id', '=','users.user_group')->where('id',$record_id)->first();
            $new_record = User::where('id',$record_id)->first();
            return response()->json([
                'status' => 'success', 'result' => $new_record,'message' => "Profile Image Successfully"], 200);
        }else{
             return response()->json([
                'status' => 'fail', 'result' => "",'message' => "Profile Image Not Updated"], 200);
        }
         
        
    }

    public function deleteUser(Request $request)
    { 
        $record_id = $request->input('id');
        $dataModel = User::where('id',$record_id)->delete();
        if($dataModel){
            return response()->json([
                'status' => 'success', 'result' => "",'message' => "User Deleted"], 200);
        }else{
         return response()->json([
            'status' => 'fail', 'result' => "",'message' => "User Not Deleted"], 200);
     }
 }

 // users ends

 /// user group start
 public function getAllUserGroups(){
        $dataList = UserGroup::all();
        if($dataList){
            return response()->json([
                'status' => 'success', 'result' => $dataList], 200);
        }else{
             return response()->json([
                'status' => 'fail', 'result' => ""], 200);
        }
    }
    public function saveUserGroup(Request $request)
    { 
         $group_name = $request->input('group_name');
        $description = $request->input('description');
        $check_existence = UserGroup::where('group_name',$group_name)->where('description',$description)->first();
        if($check_existence){// check for existence before saveing
            return response()->json([
                'status' => 'fail', 'result' => "",'message' => "UserGroup  Already Exist"], 200);
        }else{// save and return data
        $dataModel = new UserGroup();
        $dataModel->group_name = $group_name;
        $dataModel->description = $description;
        $dataModel->updated_at = Carbon::now();
        $dataModel->created_at = Carbon::now();
        $dataModel->save();
         if($dataModel){
            $new_record =  $check_existence = UserGroup::where('group_name',$group_name)->where('description',$description)->first();
            return response()->json([
                'status' => 'success', 'result' => $new_record,'message' => "User Group Saved Successfully"], 200);
        }else{
             return response()->json([
                'status' => 'fail', 'result' => "",'message' => "User Group Not Saved"], 200);
        }
        }
        
    }

    public function updateUserGroup(Request $request)
    { 
         $group_name = $request->input('group_name');
        $description = $request->input('description');
        $record_id = $request->input('user_group_id');
        $check_existence = UserGroup::where('group_name',$group_name)->where('description',$description)->first();
        if($check_existence){// check for existence before saveing
            return response()->json([
                'status' => 'fail', 'result' => "",'message' => "Record Already Exist With Details Provided"], 200);
        }else{// save and return data
        // $dataModel =  PageGroup::where('page_group_id',$record_id)->first();
       $dataModel = DB::table('user_groups')->where('user_group_id',$record_id)->update(
            [
             'group_name' => $group_name,
             'description' => $description,
             
             'updated_at' => Carbon::now(),
         ]
        );
        
         if($dataModel){
            $new_record = UserGroup::where('user_group_id',$record_id)->first();
            return response()->json([
                'status' => 'success', 'result' => $new_record,'message' => "User Group Updated Successfully"], 200);
        }else{
             return response()->json([
                'status' => 'fail', 'result' => "",'message' => "User Group Not Updated"], 200);
        }
        }
         
        
    }

    public function deleteUserGroup(Request $request)
    { 
        $record_id = $request->input('user_group_id');
        $dataModel = UserGroup::where('user_group_id',$record_id)->delete();
        if($dataModel){
            return response()->json([
                'status' => 'success', 'result' => "",'message' => "User Group Deleted"], 200);
        }else{
         return response()->json([
            'status' => 'fail', 'result' => "",'message' => "User Group Not Deleted"], 200);
     }
 }

 // user group ends

 /// user group permission start
    

  public function getThisUserGroupPermissions(Request $request){
        $record_id = $request->input('user_group_id');
         $dataList = UserGroupPermission::where('user_group',$record_id)->get();
        // $dataList = User::all();
        if($dataList){
            return response()->json([
                'status' => 'success', 'result' => $dataList], 200);
        }else{
             return response()->json([
                'status' => 'fail', 'result' => ""], 200);
        }
    }

    public function saveUserGroupPermissions(Request $request){
        $record_id = $request->input('user_group_id');
        $functions = $request->input('functions');
        $dataModel = DB::table('user_group_permissions')->where('user_group',$record_id)->delete();
        
            foreach ($functions as $row) {
            // echo $row."<br/>";
                $dataModel = new UserGroupPermission();
                $dataModel->user_group = $record_id;
                $dataModel->page_function = $row;
                $dataModel->updated_at = Carbon::now();
                $dataModel->created_at = Carbon::now();
                $dataModel->save();
            }
        
        
           $dataList = UserGroupPermission::where('user_group',$record_id)->get();
        // $dataList = User::all();
           if($dataList){
            return response()->json([
                'status' => 'success', 'result' => $dataList, "message" => "User Group Permission Updated Successfully"], 200);
        }else{
           return response()->json([
            'status' => 'fail', 'result' => "", "message" => "Could Not Updated User Group Permission"], 200);
       }
   


}

 // user group permissions ends
  
}

